Package: biomformat
Version: 1.40.0
Date: 2026-03-15
Authors@R: c(
    person("Paul J.", "McMurdie",
           email = "joey711@gmail.com",
           role = c("aut", "cre")),
    person("Joseph N.", "Paulson",
           email = "joseph.paulson@yale.edu",
           role = "aut"))
License: GPL-2
Title: An interface package for the BIOM file format
Type: Package
BugReports: https://github.com/joey711/biomformat/issues
URL: https://github.com/joey711/biomformat/, http://biom-format.org/
Description: This is an R package for interfacing with the BIOM file format. This package includes basic tools for reading biom-format files, accessing and
                 subsetting data tables from a biom object (which is more complex than a single table), as well as limited support for writing a biom-object
                 back to a biom-format file. The design of this API is intended to match the python API and other tools included with the biom-format
                 project, but with a decidedly "R flavor" that should be familiar to R users. This includes S4 classes and methods, as well as extensions of
                 common core functions/methods.
Imports: jsonlite (>= 0.9.16), Matrix (>= 1.7-0)
Depends: R (>= 4.1), methods
Suggests: testthat (>= 0.10), knitr (>= 1.10), BiocStyle (>= 1.6),
        rmarkdown (>= 0.7), SummarizedExperiment, S4Vectors, tibble,
        rhdf5
VignetteBuilder: knitr
Collate: 'allClasses.R' 'allPackage.R' 'IO-methods.R' 'BIOM-class.R'
        'SE-methods.R' 'tidy-methods.R' 'validity-methods.R'
biocViews: ImmunoOncology, DataImport, Metagenomics, Microbiome
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/biomformat
git_branch: RELEASE_3_23
git_last_commit: 85208fb
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 20:54:48 UTC; biocbuild
Author: Paul J. McMurdie [aut, cre],
  Joseph N. Paulson [aut]
Maintainer: Paul J. McMurdie <joey711@gmail.com>
