###############################################
#'  This is an R package for interfacing with the biom format.
#'
#' This is an R package for interfacing with the biom-format.
#' It includes basic utilities for reading and writing BIOM format using R,
#' and native R methods for interacting with biom-data
#' that maps to functionality in other languages
#' that support biom-format, like python.
#'
#' @import methods
#' @name biom-package
#' @author Paul J. McMurdie II \email{mcmurdie@@stanford.edu}
#' @references
#'  \url{http://www.biom-format.org/}
#'  
#' @docType package
#' @keywords package
NA
###############################################
